/** 
 ****************************************************************************
 *
 * ServiceB
 *
 * @file    ServiceBBase.cpp
 *
 * @author  Charles Halliday
 *
 * @date    15 June 2010
 *
 ****************************************************************************
 */

// Include own header
#include "ServiceBBase.hpp"

#include <cppuhelper/factory.hxx>
#include <cppuhelper/implementationentry.hxx>

#include <rtl/ustring.hxx>

#include <iostream>

// Service implementation headers (required for factory methods)
#include "ServiceBImpl.hpp"

using namespace ::std;
using namespace ::rtl;
using namespace ::cppu;
using namespace ::com::sun::star;
using namespace ::com::sun::star::lang;
using namespace ::com::sun::star::uno;

namespace test {

  // Implementation of service: test.ServiceB

  static const OUString __SERVICE_NAME_ServiceBImpl =
  OUString::createFromAscii("test.ServiceB");
  static const OUString __IMPLEMENTATION_NAME_ServiceBImpl =
  OUString::createFromAscii("test.ServiceBImpl");

  /** Factory
   * @param xComponentContext component context from client
   */
  Reference< XInterface > SAL_CALL create_ServiceBImpl(
    Reference< XComponentContext > const & xComponentContext )
    SAL_THROW( () )
  {
    ServiceBImpl * self = new ServiceBImpl(xComponentContext);
    assert(self != 0);
    return Reference < XInterface >(
      static_cast< ::cppu::OWeakObject * > (self));
  }

  /** 
   * @return all service names that may be used to access this service
   */
  static Sequence< OUString > getSupportedServiceNames_ServiceBImpl()
  {
    static ::osl::Mutex lock;
    static Sequence < OUString > *pNames = 0;
    if( ! pNames )
    {
      ::osl::Guard< ::osl::Mutex > guard(lock);
      if( !pNames )
      {
        static Sequence< OUString > seqNames(1);
        seqNames.getArray()[0] = OUString(__SERVICE_NAME_ServiceBImpl);
        pNames = &seqNames;
      }
    }
    return *pNames;
  }

  /** 
   * @return implementation name
   */
  static OUString getImplementationName_ServiceBImpl()
  {
    return OUString(__IMPLEMENTATION_NAME_ServiceBImpl);
  }

  // Base Class Constructor
  ServiceBImplBase::ServiceBImplBase(
    Reference< XComponentContext > const & xComponentContext)
    : ServiceBImplHelper(m_aBaseMutex)
  {
  }

  // Base Class Destructor
  ServiceBImplBase::~ServiceBImplBase()
  {
  }

  // XMain base implementation
  /**
   * @param aArguments arguments passed to the component,
   *        e.g. the command line arguments
   * @return status: 0 for success, otherwise component dependent error code
   */
  sal_Int32 ServiceBImplBase::run(
    const ::com::sun::star::uno::Sequence< ::rtl::OUString >& aArguments )
    throw (::com::sun::star::uno::RuntimeException)
  {
    // Override to provide initialization and/or component action on load
    return 0;
  }

  // XServiceInfo base implementation

  /**
    * @return unique name of the implementation
    */
  ::rtl::OUString ServiceBImplBase::getImplementationName()
    throw (RuntimeException)
  {
    return getImplementationName_ServiceBImpl();
  }  // ! getImplementationName

  /**
    * @param full name of service to be tested
    *
    * @return true, if service is supported, false otherwise
    */
  sal_Bool ServiceBImplBase::supportsService( OUString const & serviceName )
    throw (RuntimeException)
  {
    sal_Bool bSupportService = sal_False;
    Sequence< OUString > supportedServices = getSupportedServiceNames();
    for ( sal_Int32 i = 0; i < supportedServices.getLength(); i++) {
      if (serviceName.equals(supportedServices[i])) {
        bSupportService = sal_True;
        break;
      }
    }
    return bSupportService;
  }  // ! supportsService

  /**
    *  Provides the supported service names of the implementation
    *
    * @return service names that are supported
    */
  Sequence< OUString > ServiceBImplBase::getSupportedServiceNames()
    throw (RuntimeException)
  {
    return getSupportedServiceNames_ServiceBImpl();
  }

  // end service test.ServiceB

  // Table for UNO component implementation helper functions
  static struct ImplementationEntry s_component_entries [] =
    {
      // Table entries for helper classes.
      // One entry per service implementation
      {
        create_ServiceBImpl,
        getImplementationName_ServiceBImpl,
        getSupportedServiceNames_ServiceBImpl,
        createSingleComponentFactory,
        0,
        0,
      },
      { 0, 0, 0, 0, 0, 0 }
    };

} // namespace close



// Standard C functions required by UNO for component
extern "C"
{

  void SAL_CALL component_getImplementationEnvironment(
    sal_Char const ** ppEnvTypeName, uno_Environment ** ppEnv )
  {
    *ppEnvTypeName = CPPU_CURRENT_LANGUAGE_BINDING_NAME;
  }

  sal_Bool SAL_CALL component_writeInfo(lang::XMultiServiceFactory * xMgr,
                                        registry::XRegistryKey * xRegistry )
  {
    // Table driven helper in cppuhelper
    return component_writeInfoHelper(xMgr,
                                     xRegistry,
                                     ::test::s_component_entries );
  }

  void * SAL_CALL component_getFactory(
    sal_Char const * implName, lang::XMultiServiceFactory * xMgr,
    registry::XRegistryKey * xRegistry )
  {
    // Table driven helper in cppuhelper
    return component_getFactoryHelper(implName,
                                      xMgr,
                                      xRegistry,
                                      ::test::s_component_entries );
  }

}
